# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: report fields.

This script then assumes Solr is at https://localhost:8984/ (with a self-
signed certificate). You can override this, of course.

This reports all requested fields. Note that some fields are search-only
and cannot return values.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging, csv

_logger = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument('--query', default='*:*', help='Optional constrain query, default %(default)s')
    parser.add_argument('core', help='What Solr core to use (collections, datasets, files, etc.)')
    parser.add_argument('field', nargs='+', help='Fields to include')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr = pysolr.Solr(args.solr + args.core, always_commit=True, verify=False)

    writer = csv.writer(sys.stdout)
    writer.writerow(args.field)

    for match in find_documents(solr, args.query, args.field):
        row = []
        for field in args.field:
            value = match.get(field, None)
            if value is None:
                value = '«unknown»'
            if isinstance(value, list):
                value = ','.join(value)
            row.append(value)
        writer.writerow(row)

    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: fix the misspelled "mass spectrometry" in the LabCAS cores.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging


_logger = logging.getLogger(__name__)

__doc__ = 'Fix the misspelled "mass spectrometry" in the LabCAS cores.'


def _replace_values(solr: pysolr.Solr, dryrun: bool):
    count, replaced = 0, 0
    for match in find_documents(solr, 'DataCategory:Mass Spectometry', None):
        current = set(match.get('DataCategory'))
        if 'Mass Spectometry' in current:
            current.remove('Mass Spectometry')
            current.add('Mass Spectrometry')
            match['DataCategory'] = list(current)

            if dryrun:
                _logger.info('Dry run: would be replacing document %s with new %s', match['id'], 'DataCategory')
            else:
                _logger.debug('Replacing document %s', match['id'])
                if '_version_' in match:
                    del match['_version_']
                solr.add([match])
            replaced += 1

        count += 1
        if count % 100 == 0:
            _logger.info('Processed %d documents', count)

    _logger.info('Done! After %d documents I replaced the Data Category in %d of them', count, replaced)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make any changes but show them instead"
    )
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr_url = args.solr if args.solr.endswith('/') else args.solr + '/'
    for core_name in ['collections', 'datasets', 'files']:
        _logger.info('Processing %s', core_name)
        solr = pysolr.Solr(solr_url + core_name, always_commit=True, verify=False)
        _replace_values(solr, args.dryrun)
    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: delete field.

This script then assumes Solr is at https://localhost:8984/ (with a self-
signed certificate). You can override this, of course.

This will go through documents and delete a named field.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging

_logger = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument('core', help='What Solr core to use (collections, datasets, files, etc.)')
    parser.add_argument('field', help='Field to delete')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr = pysolr.Solr(args.solr + args.core, always_commit=True, verify=False)

    query_field, count = args.field.replace(':', '\\:'), 0
    for result in find_documents(solr, f'{query_field}:*', None):
        del result[args.field]
        del result['_version_']
        solr.add([result])
        count += 1
        if count % 100 == 0:
            _logger.info('Completed %d documents', count)
    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''Make numerous symlinks separated into validation and discovery folders for BBD
and DCIS images.

1. Run this before publishing to get the symlinks in the right place so publishing
can actually publish them.
2. Then, run the publication pipeline.
3. Then, run bbd_dcis_population to add the fields to Solr from the Excel files.

Typical invocation:

cd /usr/local/labcas/Documents/Clients/JPL/Cancer/LabCAS/Development/jpl.labcas.utils
git pull
.venv/bin/pip install --editable .
.venv/bin/restructure-bbd-dcis bbd.csv /labcas-data/labcas-backend/archive/edrn/BBD_Pathology_Slide_Images
'''

import logging, argparse, csv, os, glob

from .argparse import add_logging_argparse_options

_logger = logging.getLogger(__name__)


def _unlink(path):
    try:
        os.unlink(path)
    except FileNotFoundError:
        pass


def _found(filename: str, source_dir: str) -> str | None:
    '''Return the first matching partial `filename` found in `source_dir`.
    
    If no match is found, return `None`.
    '''
    for dirpath, _, filenames in os.walk(source_dir):
        if 'Source' in dirpath:
            _logger.debug('Skipping directory %s', dirpath)
            continue
        pattern = os.path.join(dirpath, f"{filename}.*")
        matches = glob.glob(pattern)
        if matches:
            _logger.debug('🔍 Found %r in %s', matches, dirpath)
            return matches[0]
    return None


def _make_symlinks(spreadsheet: str, target_dir: str, dryrun: bool):
    with open(spreadsheet, 'r') as io:
        reader = csv.reader(io)
        for row in reader:
            # Skip the header and the weird row with SECTION NOT AVAILABLE in it
            if row[8] == 'RefSetGroup' or row[0] == 'SECTION NOT AVAILABLE': continue
            f = _found(row[4], target_dir)
            if f is None:
                _logger.info('🤷 No match found for %s', row[4])
                continue
            is_discovery = row[8] == 'Discovery'
            destination = os.path.join(
                target_dir, 'Discovery' if is_discovery else 'Validation', os.path.basename(f)
            )
            f, destination = os.path.abspath(f), os.path.abspath(destination)
            if dryrun:
                _logger.info('🌵 Dry run; would have symlinked %s to %s', f, destination)
            else:
                _logger.info('🔗 Symlinking %s to %s', f, destination)
                _unlink(destination)
                os.symlink(f, destination)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_logging_argparse_options(parser)
    parser.add_argument('spreadsheet', type=str, help='The BBD or DCIS spreadsheet to process')
    parser.add_argument('target_dir', type=str, help='Where to find and also make symlinks')
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make any changes but show them instead"
    )

    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(asctime)s %(levelname)s %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
    os.makedirs(os.path.join(args.target_dir, 'Validation'), exist_ok=True)
    os.makedirs(os.path.join(args.target_dir, 'Discovery'), exist_ok=True)
    _logger.info('🔍 Processing spreadsheet: %s', args.spreadsheet)
    _make_symlinks(args.spreadsheet, args.target_dir, args.dryrun)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: Solr.

Various conveniences to Apache Solr.
'''

import pysolr, logging
from typing import Generator

_logger = logging.getLogger(__name__)
_rows = 10000


def find_documents(solr: pysolr.Solr, query: str, fields: list[str]) -> Generator[dict, None, None]:
    start = 0
    while True:
        if fields:
            results = solr.search(q=query, start=start, rows=_rows, fl=','.join(fields))
        else:
            results = solr.search(q=query, start=start, rows=_rows)
        num_results = len(results)
        if num_results == 0:
            _logger.debug('At start %d with rows %d got zero results for %s', start, _rows, query)
            return
        start += num_results
        for result in results:
            yield result



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: generate annual report which provides

- Number of collections
- Number of datasets
- Number of files
- Total size of files in bytes, kilobytes, megabytes, gigabytes, terabytes

for both EDRN and MCL and since a given date.

Note: this doesn't work.
'''

from . import VERSION
from .const import DEFAULT_SOLR_URL
from .argparse import add_logging_argparse_options
from .solr import find_documents
from datetime import date
import argparse, sys, pysolr, logging


_logger = logging.getLogger(__name__)
_mcl_solr_url = 'https://localhost:8985/solr/'


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('--version', action='version', version=VERSION)
    add_logging_argparse_options(parser)
    parser.add_argument(
        '-e', '--edrn', metavar='EDRN', default=DEFAULT_SOLR_URL, help='EDRN Solr URL (default %(default)s)'
    )
    parser.add_argument(
        '-m', '--mcl', metavar='MCL', default=_mcl_solr_url, help='MCL Solr URL (default %(default)s)'
    )
    parser.add_argument('DATE', help='Date to report on (YYYY-MM-DD)')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')

    edrn_url = args.edrn if args.edrn.endswith('/') else args.edrn + '/'
    mcl_url = args.mcl if args.mcl.endswith('/') else args.mcl + '/'
    since = date.fromisoformat(args.DATE)
    for label, url in (('EDRN', edrn_url), ('MCL', mcl_url)):
        report_collections(label, url, since)
        report_datasets(label, url, since)
        report_files(label, url, since)

    sys.exit(0)


def report_collections(url, since):

    edrn_solr = pysolr.Solr(edrn_url + 'files', always_commit=True, verify=False)
    mcl_solr = pysolr.Solr(mcl_url + 'files', always_commit=True, verify=False)

    since = date.fromisoformat(args.DATE)
    edrn_collections = find_collections(edrn_solr, since)
    mcl_collections = find_collections(mcl_solr, since)

    edrn_datasets = find_datasets(edrn_solr, since)
    mcl_datasets = find_datasets(mcl_solr, since)

    edrn_files = find_files(edrn_solr, since)
    mcl_files = find_files(mcl_solr, since)


    total, count = 0, 0
    for match in find_documents(solr, args.query, ['id', 'FileSize']):
        count += 1
        ident, size_str = match.get('id', '«unknown»'), match.get('FileSize', None)
        if size_str is None:
            _logger.warning('File «%s» has no FileSize; ignoring', ident)
            continue
        total += int(size_str)
        if count % 1000 == 0:
            _logger.info('Processed %d files, size so far is %d GB', count, total/1024/1024/1024)

    print(f'Number of files: {count}')

    kilobytes = int(total / 1024)
    megabytes = int(total / 1024 / 1024)
    gigabytes = int(total / 1024 / 1024 / 1024)
    terabytes = int(total / 1024 / 1024 / 1024 / 1024)
    print(
        f'Total size: {total} bytes ({kilobytes} KB, {megabytes} MB, {gigabytes} GB,'
        f' {terabytes} TB)'
    )
    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: argument parsing.'''


from . import VERSION
from .const import DEFAULT_SOLR_URL
import logging


def add_logging_argparse_options(parser):
    '''Add logging options to the given `parser`.'''
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-d',
        '--debug',
        action='store_const',
        const=logging.DEBUG,
        default=logging.INFO,
        dest='loglevel',
        help='Log copious debugging messages suitable for developers',
    )
    group.add_argument(
        '-q',
        '--quiet',
        action='store_const',
        const=logging.WARNING,
        dest='loglevel',
        help="Don't log anything except warnings and critically-important messages",
    )


def add_standard_argparse_options(parser):
    '''Add the regular set of expected options (logging, version, solr).'''

    # First, add the `--version` option
    parser.add_argument('--version', action='version', version=VERSION)

    # Now logging
    add_logging_argparse_options(parser)

    # Lastly, Solr URL
    parser.add_argument('-s', '--solr', metavar='URL', default=DEFAULT_SOLR_URL, help='Solr URL (default %(default)s)')



import re
import pandas as pd
import numpy as np


def decode_b_value(val):
    """
    Attempt to decode a DICOM b-value from various formats (DataElement, string, bytes, list, etc.)
    
    Parameters:
        val: The value to decode (can be a pydicom DataElement, str, bytes, list, etc.)
    
    Returns:
        float: Decoded b-value or NaN if decoding fails.
    """
    try:
        # Step 1: Get .value if it's a DataElement
        if hasattr(val, 'value'):
            val = val.value

        # Step 2: If it's a numeric type, return as float
        if isinstance(val, (int, float)):
            return float(val)

        # Step 3: If it's a string, strip and convert
        if isinstance(val, str):
            return float(val.strip())

        # Step 4: If it's bytes, decode and convert
        if isinstance(val, bytes):
            return float(val.decode('ascii').strip())

        # Step 5: If it's a list-like structure (e.g., list of ints), try to decode as characters
        if isinstance(val, (list, tuple, np.ndarray)):
            chars = ''.join(chr(int(v)) for v in val if 32 <= int(v) <= 126)
            return float(chars.strip())

    except (ValueError, TypeError) as e:
        print(f"Warning: Failed to decode b-value: {e} | Original value: {val}")

    # Step 6: If all decoding attempts fail, return NaN
    return float('nan')


def extract_bval(info):
    """
    Extract the b-value from a DICOM dataset using known tag locations.
    
    Parameters:
        info (Dataset): The DICOM dataset from which to extract the b-value.
    
    Returns:
        tuple: (bval, bval_source) where bval is a float and bval_source is a tag identifier string.
    """
    bval = float('nan')
    bval_source = ''

    # Step 1: Try to extract value using a series of known tag paths
    def try_tag(tag_path, label):
        nonlocal bval, bval_source
        val = info.get(tag_path, None)
        if val is not None:
            bval_candidate = decode_b_value(val)
            if not pd.isna(bval_candidate):
                bval = bval_candidate
                bval_source = label

    try_tag((0x0018, 0x9087), '(0018,9087)')
    if pd.isna(bval):
        try_tag((0x0019, 0x100c), '(0019,100c)')
    if pd.isna(bval):
        try_tag((0x0043, 0x1039), '(0043,1039)')
    if pd.isna(bval):
        try_tag((0x2001, 0x1003), '(2001,1003)')

    # Step 2: Fallback to parsing b-value from 'SequenceName' if available
    if pd.isna(bval) and 'SequenceName' in info:
        matches = re.findall(r'(\d+)', str(info.SequenceName))
        if matches:
            bval = float(matches[0])
            bval_source = '(0018,0024)'

    return bval, bval_source



'''🛠️ JPL LabCAS Utilities: fix event IDs in a the files core.

This will take an alias.json file and find in Solr's `files` core all
matching documents and will set the `labcasName` to have the proper underscore
with event ID as well as add the `eventID` field.

The alias.json file should already exist and is given as an command-line
argument.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging, os.path, json, re


_logger = logging.getLogger(__name__)
__doc__ = 'Fix the event IDs in the files core'
_event_id_re = re.compile(r'_([0-9]+)\.[Dd][Cc][Mm]$')


def _update(solr: pysolr.Solr, doc: dict, aliased_path: str, aliased_file_name: str, new_event_id: str):
    _logger.info('✍️ Updating %s with path %s, file name %s, and event ID %s', doc['id'], aliased_path, aliased_file_name, new_event_id)
    update = {
        'id': doc['id'],
        'labcasId': {'set': [aliased_path]},
        'FileId': {'set': [aliased_path]},
        'labcasName': {'set': [aliased_file_name]},
        'FileName': {'set': [aliased_file_name]},
        'eventID': {'set': [new_event_id]},
    }
    solr.add(update, commit=True)


def _fix_event_ids(solr: pysolr.Solr, collection: str, aliases: str, dryrun: bool = True):
    with open(aliases, 'r') as io:
        aliases = json.load(io)
    for doc in find_documents(solr, f'CollectionId:{collection}', ['id', 'labcasName', 'eventID']):
        _logger.debug('🔎 CONSIDERING %s', doc['id'])
        identifier, labcas_name, event_id = doc['id'], doc.get('labcasName', [''])[0], doc.get('eventID', [''])[0]
        if identifier not in aliases:
            _logger.info('🤐 Skipping %s because it is not in the alias.json file', identifier)
            continue
        if '_' in labcas_name and event_id is not None and event_id != '':
            _logger.info('🎉 Skipping %s because it already has a proper labcas_name %s and event ID %s', identifier, labcas_name, event_id)
            continue
        
        # Special case: the alias might just be a filename + extension, in which case still don't have to do anything
        aliased_path = aliases[identifier]
        aliased_file_name = os.path.basename(aliased_path)
        event_id_matches = _event_id_re.search(aliased_file_name)
        if event_id_matches is None:
            _logger.info('❓ Skipping %s because it does not have an event ID in the aliased file name', identifier)
            continue
        new_event_id = event_id_matches.group(1)
        if dryrun:
            _logger.info(
                '🙂‍↔️ Not updating %s with path %s, file name %s, or event ID %s since this is a dry run',
                identifier, aliased_path, aliased_file_name, new_event_id
            )
        else:
            _update(solr, doc, aliased_path, aliased_file_name, new_event_id)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make changes, show them instead"
    )
    parser.add_argument('collection', help='ID of the collection to fix')
    parser.add_argument('aliases', help='The alias.json file to use')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr_url = args.solr if args.solr.endswith('/') else args.solr + '/'
    solr = pysolr.Solr(solr_url + 'files', always_commit=True, verify=False)
    _fix_event_ids(solr, args.collection, args.aliases, args.dryrun)

    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: patch DICOM files and Solr metadata based on info from the
DMCC API.

This … not sure how this'll work
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
from .dmcc_api import dmcc_authenticate, get_image_events
from .const import DEFAULT_TOKEN_ID, DEFAULT_PROTOCOL
import argparse, sys, pysolr, logging, pydicom, os.path, os, getpass


_logger = logging.getLogger(__name__)
__doc__ = 'Add data from DMCC to DICOM files and Solr'


def _patch(doc: dict, event, solr, dryrun: bool):
    '''Patch the DICOM file and metadata in `solr` described by `doc` with the info in `event`.'''

    # `doc` is the search result from Solr and has `id` (single value), `RealFileLocation` (list of single
    # value), and `name` (list of single value).
    #
    # `event` is from the DMCC and has fields `participantId`, `siteId`, `imageEventIdentifier`, `visitTypeText`,
    # `imageTypeText`, and `protocolId`
    #
    # Assemble the DICOM filename from `RealFileLocation` and `name`.
    #
    # This is TBD!
    breakpoint()


def _patch_files_and_metadata(events: list, solr: pysolr.Solr, dryrun: bool):
    '''Patch all files and Solr metadata by using `solr` and the information in `events`.'''

    # In DMCC it's `imageEventIdentifier`, but in Solr it's `eventID`
    for event in events:
        event_id = event['imageEventIdentifier']
        found_doc = False
        for doc in find_documents(solr, f'eventID:{event_id}', ['id', 'RealFileLocation', 'name']):
            found_doc = True
            _patch(doc, event, solr, dryrun)
        if not found_doc:
            _logger.info('No documents in Solr match DMCC event ID %s', event_id)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make any changes but show them instead"
    )
    parser.add_argument('-i', '--id', default=DEFAULT_TOKEN_ID, help='Client ID; default %(default)s')
    parser.add_argument(
        '-w', '--secret', help='Client secret; will use CLIENT_SECRET env var if not given, or will prompt for it'
    )
    parser.add_argument('-p', '--protocol', default=str(DEFAULT_PROTOCOL), help='Protocol ID, defaults to %(default)s')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr_url = args.solr if args.solr.endswith('/') else args.solr + '/'
    solr = pysolr.Solr(solr_url + 'files', always_commit=True, verify=False)
    secret = args.secret
    if not secret:
        secret = os.getenv('CLIENT_SECRET')
        if not secret:
            secret = getpass.getpass('Client secret: ')
    protocol = int(args.protocol)
    token = dmcc_authenticate(args.id, secret)
    events = get_image_events(protocol, token)
    _patch_files_and_metadata(events, solr, args.dryrun)

    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: replace a field in multiple documents.

This script then assumes Solr is at https://localhost:8984/ (with a self-
signed certificate). You can override this, of course.

This will take an existing document and replace a list field with a new
items as given on the command line.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging


_logger = logging.getLogger(__name__)

__doc__ = 'Replace a field with new values in multiple documents'


def _show_identifiers(solr: pysolr.Solr, query: str):
    for match in find_documents(solr, query, ['id']):
        print(match['id'])


def _replace_values(solr: pysolr.Solr, query: str, field: str, values: list[str], dryrun: bool):
    count, replaced, proposed = 0, 0, set(values)
    for match in find_documents(solr, query, None):
        current = match.get(field, set())
        if current == proposed:
            _logger.debug('Doc %s already has expected values, skipping', match['id'])
            continue

        # Solr treats keys as case insensitive so when we try to change the case of a field
        # (such as ProtocolID → ProtocolId) it doesn't actually do anything, so let's delete
        # any fields without regard to case
        for key in match.keys():
            if key.lower() == field.lower():
                del match[key]
                break

        _logger.debug('Replacing attribute %s of doc %s with values  %r', field, match['id'], values)
        match[field] = values
        del match['_version_']

        if dryrun:
            _logger.info('Dry run: would be replacing document %s with new %s', match['id'], field)
            replaced += 1
        else:
            _logger.debug('Replacing document %s', match['id'])
            solr.add([match])
            replaced += 1

        count += 1
        if count % 100 == 0:
            _logger.info('Processed %d documents', count)

    _logger.info('Done! After %d documents I replaced %s in %d of them', count, field, replaced)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-i', '--id', action='store_true',
        help='Select IDs only rather than make any changes; all command-line arguments must still be given'
    )
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make any changes but show them instead"
    )
    parser.add_argument('core', help='What Solr core to use (collections, datasets, files, etc.)')
    parser.add_argument('query', help='Query to select documents to update')
    parser.add_argument('field', help='List field whose values to replace')
    parser.add_argument('value', nargs='+', help='New values')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr = pysolr.Solr(args.solr + args.core, always_commit=True, verify=False)

    if args.id:
        _show_identifiers(solr, args.query)
    else:
        _replace_values(solr, args.query, args.field, args.value, args.dryrun)

    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: DMCC API.'''


from .const import DMCC_TOKEN_API, DMCC_IMAGE_EVENTS
import requests


def dmcc_authenticate(client_id: str, client_secret: str) -> str:
    '''Authenticate to the DMCC's token API with a given client ID and secret, returning a
    bearer token.
    '''
    data = {
        'grant_type': 'client_credentials',
        'client_id': client_id,
        'client_secret': client_secret
    }
    headers = {'Content-type': 'application/x-www-form-urlencoded'}
    response = requests.post(DMCC_TOKEN_API, data=data, headers=headers)
    response.raise_for_status()
    return response.json().get('access_token')


def get_image_events(protocol_id: int, token: str):
    '''Retrieve all image events from the DMCC's image event API.'''
    headers = {'Authorization': f'Bearer {token}'}
    params = {'protocolId': protocol_id}
    response = requests.get(DMCC_IMAGE_EVENTS, headers=headers, params=params)
    return response.json()['imagingEventRecords']



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: constants.'''


DEFAULT_SOLR_URL    = 'https://localhost:8984/solr/'
PROTOCOL_RDF_SOURCE = 'https://edrn.jpl.nasa.gov/cancerdataexpo/rdf-data/protocols/@@rdf'
DMCC_TOKEN_API      = 'https://www.compass.fhcrc.org/edrnauthapi/auth/token'
DMCC_IMAGE_EVENTS   = 'https://www.compass.fhcrc.org/edrnrestws/Data/GetAllImageEvents'
DEFAULT_PROTOCOL    = 430  # 430 is for Prostate MRI
DEFAULT_TOKEN_ID    = '59502649ee224c2fa58362d336e4c92c'


# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: split BRSI files. BRSI stands for "Breast Reference Set
Images".

This will open and extract a GZIP'd tar file and will split the contents based
on an Excel spreadsheet (exported to CSV).

The splitting is done per:
https://github.com/EDRN/EDRN-metadata/issues/97#issuecomment-2045737597
'''

from .argparse import add_logging_argparse_options
import argparse, sys, csv, logging, dataclasses, os, tarfile


_logger = logging.getLogger(__name__)


__doc__ = 'Split BRSI archives into training and validation directories'


@dataclasses.dataclass(order=True, frozen=True)
class BRSIFile:
    '''Overkill.

    All we needed was a mapping from jpl_id to a boolean. Oh well.
    '''
    jpl_id: str = dataclasses.field(hash=True)
    training: bool = dataclasses.field(hash=False, repr=True)


def _read_spreadsheet(spreadsheet_input):
    '''Read the `spreadsheet_input` and return a mapping of `jpl_id` → `BRSIFile`.'''
    classifications = {}
    try:
        reader = csv.reader(spreadsheet_input)
        for row in reader:
            if row[0] == 'Ppt ID': continue
            training, validation, jpl_id = row[5].strip(), row[6].strip(), row[9].strip()
            training, validation = training == 'Yes', validation == 'Yes'
            if not training and not validation:
                _logger.info('File %s in the spreadsheet is neither training nor validation so ignoring', jpl_id)
            classifications[jpl_id] = BRSIFile(jpl_id, training)
    finally:
        spreadsheet_input.close()
    return classifications


def _jpl_id(filepath: str) -> str:
    '''Get the "JPL ID" out of the given `filepath`.'''
    return filepath.split('/')[-1][0:6]


def _find_files(tar: tarfile):
    '''Yield only file entries in the given `tar`.'''
    for tarinfo in tar:
        if tarinfo.isfile(): yield tarinfo


def _classify_files(tf_gen, classifications: dict):
    '''Classify the given `TarInfo`s according to the `classifications` and if found, yield the `tf`
    and whether it's training or not.

    This annotates each `TarInfo` with a boolean (`True` = training, `False` = validation).
    '''
    for tf in tf_gen:
        classification = classifications.get(_jpl_id(tf.name))
        if classification:
            yield tf, classification.training
        else:
            _logger.info('Tar file %s not found in spreadsheeet, ignoring', tf.name)


def _filter_training_files(annotated_tf_gen, training):
    '''Yield only those annotated `TarInfo`s that match the boolean `training`.'''
    for tf, was_training in annotated_tf_gen:
        if was_training == training:
            yield tf


def _sort_files(classifications: dict, archive: argparse.FileType, training: str, validation: str):
    '''Extract files from the given `archive` and sort them into `training` and `validation`
    directories based on the `classifications`.
    '''
    with tarfile.open(mode='r', fileobj=archive) as tar:
        tar.extractall(
            path=training,
            members=_filter_training_files(_classify_files(_find_files(tar), classifications), training=True)
        )
        tar.extractall(
            path=validation,
            members=_filter_training_files(_classify_files(_find_files(tar), classifications), training=False)
        )


def main():
    '''Do it.'''
    parser = argparse.ArgumentParser(description=__doc__)
    add_logging_argparse_options(parser)
    parser.add_argument('csv', type=argparse.FileType('r'), help='Spreadsheet')
    parser.add_argument('archive', type=argparse.FileType('rb'), help="GZIP'd archive file")
    parser.add_argument('target', help='Target directory')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    os.makedirs(args.target, exist_ok=True)
    training, validation = os.path.join(args.target, 'training'), os.path.join(args.target, 'validation')
    os.makedirs(training, exist_ok=True)
    os.makedirs(validation, exist_ok=True)
    classifications = _read_spreadsheet(args.csv)
    _sort_files(classifications, args.archive, training, validation)
    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: replace field.

This script then assumes Solr is at https://localhost:8984/ (with a self-
signed certificate). You can override this, of course.

This will take an existing document and replace a singly-valued list
field with a new single-value in a singly-valued list.
'''

from .argparse import add_standard_argparse_options
import argparse, sys, pysolr, logging
from typing import Iterable

_logger = logging.getLogger(__name__)


def _first(i: Iterable) -> object:
    iterator = iter(i)
    return next(iterator)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument('core', help='What Solr core to use (collections, datasets, files, etc.)')
    parser.add_argument('id', help='ID of the document to update')
    parser.add_argument('field', help='List field whose values to replace')
    parser.add_argument('value', help='New single-item list value to replace with')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr = pysolr.Solr(args.solr + args.core, always_commit=True, verify=False)

    # Now gather up the EDRN collections
    results = solr.search(q=f'id:{args.id}')
    if results.hits != 1:
        print(f'Unexpected number of hits for «{args.id}»: {results.hits}; aborting')
        sys.exit(-1)

    match = _first(results)
    match[args.field] = [args.value]
    del match['_version_']
    # Curiously, we don't have to delete the doc first … just adding it works to overwrite
    solr.add([match])
    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: substitute the value of a field in multiple documents.

This takes a Solr URL, a core name, a field name, an existing value, and a replacement value.

For example:

    sub-field --url https://localhost:8987/solr/ files OwnerPrincipal cn=Feng cn=Zhang

will replace all occurrences of `cn=Feng` with `cn=Zhang` in the `OwnerPrincipal` field of all documents in the `files` core.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging


_logger = logging.getLogger(__name__)
__doc__ = 'Substitute the value of a field in multiple documents'
_batch_size = 100


def _replace(solr: pysolr.Solr, batch: list[dict], dryrun: bool):
    if dryrun:
        _logger.info('🌵 Dry run: would be replacing %d documents', len(batch))
    else:
        _logger.info('🔄 Replacing %d documents', len(batch))
        solr.add(batch)


def _substitute_values(solr: pysolr.Solr, field: str, old: str, new: str, dryrun: bool):
    count, replaced, batch, query = 0, 0, [], f'{field}:"{old}"'
    for match in find_documents(solr, query, ['OwnerPrincipal', 'id']):
        count += 1
        _logger.debug('Replacing attribute %s of doc %s with value %s', field, match['id'], new)
        doc_id, op = match['id'], match.get(field)
        if op is None:
            _logger.warning('🚨 Document %s has no %s field; skipping', doc_id, field)
            continue
        if isinstance(op, list):
            new_values = [new if i == old else i for i in op]
            batch.append({'id': doc_id, field: {'set': new_values}})
            replaced += 1
        elif isinstance(op, str):
            batch.append({'id': doc_id, field: {'set': new}})
            replaced += 1
        else:
            _logger.warning('🚨 Document %s has an unexpected %s field of type %s; skipping', doc_id, field, type(op))
            continue

        if replaced >= _batch_size:
            _replace(solr, batch, dryrun)
            batch, replaced = [], 0

        if count % 100 == 0:
            _logger.info('Processed %d documents', count)

    if len(batch) > 0:
        _replace(solr, batch, dryrun)

    _logger.info('Done after %d documents', count)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make any changes but show them instead"
    )
    parser.add_argument('core', help='What Solr core to use (collections, datasets, files, etc.)')
    parser.add_argument('field', help='List field whose values to replace')
    parser.add_argument('old', help='Old value to replace')
    parser.add_argument('new', help='New value to replace with')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr_url = args.solr if args.solr.endswith('/') else args.solr + '/'
    solr = pysolr.Solr(solr_url + args.core, always_commit=True, verify=False)

    _substitute_values(solr, args.field, args.old, args.new, args.dryrun)

    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''Populate the BBD and DCIS collections in Solr with with the fields from the CSV files.

Typical invocation: `rm -f nohup.out; nohup .venv/bin/populate-bbd-dcis bbd.csv bbd &`
'''

import logging, argparse, csv, os, pysolr

from .argparse import add_standard_argparse_options

_logger = logging.getLogger(__name__)


_shortcut_to_collection_id = {
    'bbd': 'BBD_Pathology_Slide_Images',
    'dcis': 'DCIS_Pathology_Slide_Images',
}
_shortcut_to_field_name = {
    'bbd': 'New BBD ID',
    'dcis': 'New DCIS ID',
}

def _populate_solr(spreadsheet: str, solr: pysolr.Solr, collection_shortcut: str, dryrun: bool):
    cid = _shortcut_to_collection_id[collection_shortcut]
    field_name = _shortcut_to_field_name[collection_shortcut]

    _logger.debug(
        '🧑‍🧑‍🧒‍🧒 Populating Solr at %s with "%s" for collection "%s", field "%s", dryrun=%r',
        solr.url, spreadsheet, cid, field_name, dryrun
    )

    with open(spreadsheet, 'r') as io:
        reader = csv.reader(io)
        for row in reader:
            # Skip the header and the weird row with SECTION NOT AVAILABLE in it
            if row[8] == 'RefSetGroup' or row[0] == 'SECTION NOT AVAILABLE': continue
            fn = row[4]  # Does HK want any other fields?
            query = f'CollectionId:"{cid}" AND (FileName:{fn}.svs OR FileName:{fn}.scn)'
            results = solr.search(query, fl=['id'], start=0, rows=9999)
            if len(results) == 0:
                _logger.warning('⚠️ No results for %s', query)
                continue
            for doc in results:
                _logger.info('✍️ Updating %s with "%s"', doc['id'], fn)
                update = {'id': doc['id'], field_name: {'set': [fn]}}
                if dryrun:
                    _logger.info(
                        '🌵 Dry run; would have updated %s field %s set to "%s"',
                        doc['id'], field_name, fn
                    )
                else:
                    solr.add(update, commit=True)
                    _logger.info(
                        '✅ Updated %s field %s set to "%s"', doc['id'],
                        field_name, fn
                    )


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make any changes but show them instead"
    )
    parser.add_argument('spreadsheet', help='The CSV file with the fields to use to populate.')
    parser.add_argument(
        'collection_shortcut', help='What collection to update', choices=['bbd', 'dcis']
    )
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr_url = args.solr if args.solr.endswith('/') else args.solr + '/'
    solr = pysolr.Solr(solr_url + 'files', always_commit=True, verify=False)
    _populate_solr(args.spreadsheet, solr, args.collection_shortcut, args.dryrun)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''Find the missing anonymized BBD or DCIS images from a given spreadsheet by traversing
the /labcas-data/amahabal/HandE_Images_from_KUMC directory.

This uses the `E` column of the spreadsheet since apparenty the files are named
after the "New BBD ID" or "New DCIS ID" column (column 4 after zero index).
'''

import logging, argparse, csv, os, glob

from .argparse import add_logging_argparse_options

_logger = logging.getLogger(__name__)


def _found(filename: str, source_dir: str) -> bool:
    for dirpath, _, filenames in os.walk(source_dir):
        pattern = os.path.join(dirpath, f"{filename}.*")
        matches = glob.glob(pattern)
        if matches:
            _logger.debug('🔍 Found %s.* in %s', filename, dirpath)
            return True
    return False


def _find_missing_images(spreadsheet: str, source_dir: str):
    with open(spreadsheet, 'r') as io:
        reader = csv.reader(io)
        for row in reader:
            # Skip the header and the weird row with SECTION NOT AVAILABLE in it
            if row[8] == 'RefSetGroup' or row[0] == 'SECTION NOT AVAILABLE': continue
            if not _found(row[4], source_dir): print(row[4])


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_logging_argparse_options(parser)
    parser.add_argument('spreadsheet', type=str, help='The BBD or DCIS spreadsheet to process')
    parser.add_argument('source_dir', type=str, help='Where to find the original DICOM files')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(asctime)s %(levelname)s %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
    _logger.info('🔍 Processing spreadsheet: %s', args.spreadsheet)
    _find_missing_images(args.spreadsheet, args.source_dir)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities.'''

import importlib.resources


PACKAGE_NAME = __name__
__version__ = VERSION = importlib.resources.files(__name__).joinpath('VERSION.txt').read_text().strip()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: fix patient IDs in "Lung Team Project 2" `.dcm` files for 

This queries Solr for all files matching a given query and replaces the DICOM patient ID
header with the event ID as dictated by Solr.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging, pydicom, os.path


_logger = logging.getLogger(__name__)
__doc__ = 'Replace patient IDs in DICOM files with event IDs'


def _replace_patient_IDs(solr: pysolr.Solr, query: str, dryrun: bool):
    count = 0
    for doc in find_documents(solr, query, ['id', 'RealFileLocation', 'name', 'eventID']):
        doc_id, event_id = doc['id'], doc.get('eventID', [None])[0]
        dn, fn = doc.get('RealFileLocation', [None])[0], doc.get('name', [None])[0]
        count += 1
        if event_id is None:
            _logger.warning('No "eventID" for %s; skipping', event_id)
            continue
        if dn is None:
            _logger.warning('No "RealFileLocation" for %s; skipping', doc_id)
            continue
        if fn is None:
            _logger.warning('No "name" for %s; skipping', doc_id)
            continue
        actual_file = os.path.join(dn, fn)
        if not os.path.exists(actual_file):
            _logger.warning('Solr doc %s, file %s, does not exist; skipping', doc_id, actual_file)
            continue
        elif os.path.isdir(actual_file):
            _logger.warning('Solr doc %s, file %s, is actually a directory; skipping', doc_id, actual_file)
        else:
            try:
                dcm = pydicom.dcmread(actual_file)
                if dryrun:
                    _logger.info('DRY RUN NOT PATCHING %s', actual_file)
                else:
                    if dcm.PatientID != event_id:
                        _logger.info('PATCHING %s with event ID %s', actual_file, event_id)
                        dcm.PatientID = event_id
                        dcm.save_as(actual_file)
            except Exception as ex:
                _logger.warning('Exception %s reading/writing %s for Solr doc %s; skipping', ex, actual_file, doc_id)
                continue
        if count % 1000 == 0:
            _logger.info('Processed %d Solr entries so far', count)

    _logger.info('Total Solr entries retrieved: %d', count)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make any changes but show them instead"
    )
    parser.add_argument('query', help='Solr query to select files to update')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr_url = args.solr if args.solr.endswith('/') else args.solr + '/'
    solr = pysolr.Solr(solr_url + 'files', always_commit=True, verify=False)

    _replace_patient_IDs(solr, args.query, args.dryrun)
    sys.exit(0)

    # Example of overwriting a field in a DICOM file:
    #
    # fn = '1097_ser005img074_CTThoraxHigh_Lung.dcm'
    # try:
    #     dataset = pydicom.dcmread(fn)
    #     print('Before: Patient ID = ', dataset.PatientID)
    #     dataset.PatientID = args.query
    #     dataset.save_as(fn)
    # except Exception as ex:
    #     breakpoint()
    #     _logger.exception(ex)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: date report.

This script then assumes Solr is at https://localhost:8984/ (with a self-
signed certificate). You can override this, of course.

This will write a CSV file with each collection and the known dates.
'''

from .argparse import add_standard_argparse_options
from .const import PROTOCOL_RDF_SOURCE
from .rdf import read_rdf
from urllib.parse import urlparse
import argparse, sys, rdflib, pysolr, csv, logging

_logger = logging.getLogger(__name__)


# RDF type for protocol objects
_protocol_type = rdflib.URIRef('http://edrn.nci.nih.gov/rdf/types.rdf#Protocol')

# RDF predicate for the start date
_start_date = rdflib.URIRef('http://edrn.nci.nih.gov/rdf/schema.rdf#startDate')

# RDF predicate for the estimated finish date
_estimated_finish_date = rdflib.URIRef('http://edrn.nci.nih.gov/rdf/schema.rdf#estimatedFinishDate')

# RDF predicate for the finish date
_finish_date = rdflib.URIRef('http://edrn.nci.nih.gov/rdf/schema.rdf#finishDate')


def _dmcc_id(subject: rdflib.URIRef) -> str:
    '''Convert an RDF subject URI (specifically a URL in this case) into a DMCC protocol ID.
    I.E, convert `http://edrn.nci.nih.gov/data/protocols/237` → `237`
    '''
    return urlparse(subject).path.split('/')[-1]


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-p', '--protocols', metavar='URL', default=PROTOCOL_RDF_SOURCE,
        help='Where to find RDF for protocols (default %(default)s)'
    )
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr, statements, protocols = pysolr.Solr(args.solr + 'collections', verify=False), read_rdf(args.protocols), {}

    # First, gather up the title and the start, estimated finish, and finish dates of all protocols
    for subject_uri, predicates in statements.items():
        kind = predicates.get(rdflib.RDF.type, [None])[0]
        if kind != _protocol_type: continue
        start = predicates.get(_start_date, ['«unknown»'])[0]
        estimated_finish = predicates.get(_estimated_finish_date, ['«unknown»'])[0]
        finish = predicates.get(_finish_date, ['«unknown»'])[0]
        title = predicates.get(rdflib.DCTERMS.title, ['«unknown title»'])[0]
        protocols[_dmcc_id(subject_uri)] = (title, start, estimated_finish, finish)

    # Now gather up the EDRN collections
    results = solr.search(
        q='Consortium:EDRN', rows=999999,
        fl=['Date', 'ProtocolId', 'DateDatasetFrozen', 'DateProductFrozen', 'CollectionId']
    )

    # Go through the resuts and write each row
    writer = csv.writer(sys.stdout)
    writer.writerow([
        'Collection ID', 'LabCAS Date', 'LabCAS Date Dataset Frozen', 'LabCAS Date Product Frozen',
        'Protocol ID', 'Protocol Title', 'Protocol Start', 'Estimated Protocol Finish', 'Protocol Finish'
    ])
    for result in results:
        collection_id = result['CollectionId'][0]
        protocol_id = result.get('ProtocolId')
        if protocol_id is None:
            _logger.warning('Collection %s does not have a protocol ID; skipping', collection_id)
            continue
        protocol_id = protocol_id[0]
        pdetails = protocols.get(protocol_id)
        if not pdetails:
            _logger.warning('Collection %s refers to unknown protocol ID %s; skipping', collection_id, protocol_id)
            continue

        row = [
            collection_id,
            result.get('Date', ['«unknown»'])[0],
            result.get('DateDatasetFrozen', ['«unknown»'])[0],
            result.get('DateProductFrozen', ['«unknown»'])[0],
            protocol_id,
            pdetails[0],
            pdetails[1],
            pdetails[2],
            pdetails[3]
        ]
        writer.writerow(row)

    sys.stdout.close()
    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: delete an entire collection.

It'll also delete all datasets and files under that collection.
'''

from .argparse import add_standard_argparse_options
import argparse, sys, pysolr, logging

_logger = logging.getLogger(__name__)


def _delete_files(collectionid: str, dryrun: bool, solr: pysolr.Solr):
    q = f'CollectionId:{collectionid}'
    if dryrun:
        logging.info('Dry run; would be calling delete on files with q=%s', q)
    else:
        logging.info('Deleting files matching q=%s', q)
        solr.delete(q=q)


def _delete_datasets(collectionid: str, dryrun: bool, solr: pysolr.Solr):
    q = f'CollectionId:{collectionid}'
    if dryrun:
        logging.info('Dry run; would be calling delete on datasets with q=%s', q)
    else:
        logging.info('Deleting datasets matching q=%s', q)
        solr.delete(q=q)


def _delete_collection(collectionid: str, dryrun: bool, c: pysolr.Solr, d: pysolr.Solr, f: pysolr.Solr):
    _delete_files(collectionid, dryrun, f)
    _delete_datasets(collectionid, dryrun, d)
    if dryrun:
        logging.info('Dry run; would be calling delete on collections with id=«%s»', collectionid)
    else:
        logging.info('Deleting collection «%s»', collectionid)
        c.delete(id=collectionid)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make any changes but show them instead"
    )
    parser.add_argument(
        '-c', '--confirm', action='store_true', help="Confirmed: don't ask for confirmation; hope you made backups"
    )
    parser.add_argument('collectionid', help='ID of the collection to delete (the id field)')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr = args.solr if args.solr.endswith('/') else args.solr + '/'
    c = pysolr.Solr(solr + 'collections', always_commit=True, verify=False)
    d = pysolr.Solr(solr + 'datasets', always_commit=True, verify=False)
    f = pysolr.Solr(solr + 'files', always_commit=True, verify=False)

    if not args.dryrun:
        if not args.confirm:
            confirmation = input('This is a highly destructive action and hope you have backups! Proceed? ')
            if confirmation.lower() != 'yes':
                logging.warning('Aborted because user did not confirm')
                sys.exit(-1)

    _delete_collection(args.collectionid, args.dryrun, c, d, f)

    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: report total size in bytes of all files.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging

_logger = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        'query', help='Query to select files to report on; default "%(default)s"',
        default='Consortium:EDRN'
    )
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr = pysolr.Solr(args.solr + 'files', always_commit=True, verify=False)

    total, count = 0, 0
    for match in find_documents(solr, args.query, ['id', 'FileSize']):
        count += 1
        ident, size_str = match.get('id', '«unknown»'), match.get('FileSize', None)
        if size_str is None:
            _logger.warning('File «%s» has no FileSize; ignoring', ident)
            continue
        total += int(size_str)
        if count % 1000 == 0:
            _logger.info('Processed %d files, size so far is %d GB', count, total/1024/1024/1024)

    print(f'Number of files: {count}')

    kilobytes = int(total / 1024)
    megabytes = int(total / 1024 / 1024)
    gigabytes = int(total / 1024 / 1024 / 1024)
    terabytes = int(total / 1024 / 1024 / 1024 / 1024)
    print(
        f'Total size: {total} bytes ({kilobytes} KB, {megabytes} MB, {gigabytes} GB,'
        f' {terabytes} TB)'
    )
    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''Show usage of DICOM headers in Lung_Team_Project_2 and Prostate_MRI collections.

See https://github.com/jpl-labcas/jpl.labcas.utils/issues/2 for more details.
'''

from . import VERSION
from .argparse import add_logging_argparse_options
from pydicom.errors import InvalidDicomError, BytesLengthException
from pydicom.sequence import Sequence
from pydicom.tag import Tag
from pydicom import datadict
from typing import Generator, Dict, Any
import argparse, logging, sys, os, time, humanize, pydicom, tempfile, gdown, csv, collections, re, multiprocessing, random


_logger = multiprocessing.log_to_stderr()
_default_spreadsheet_id = '1Q56vKzK0nB4UAkfLJnBOy6C-7wtHccvZkWYGQHTMpBw'
_site_re = re.compile(r'(Images_Site_[A-Za-z0-9]+|LTP2-Site6)')


class ScannedTag:
    '''Tag for a scanned DICOM file.'''
    def __init__(self, tag: Tag):
        self.tag = tag
        self.in_file = self.in_spreadsheet = False

    def __hash__(self):
        return hash(self.tag)
    
    def __eq__(self, other):
        return self.tag == other.tag

    def __lt__(self, other):
        return self.tag < other.tag

    def __str__(self):
        return f'{self.tag} in file: {self.in_file}, in spreadsheet: {self.in_spreadsheet}'

    def __repr__(self):
        return f'<{self.__class__.__name__}(tag={self.tag},in_file={self.in_file},in_spreadsheet={self.in_spreadsheet})>'


def _load_hk_spreadsheet(spreadsheet_id: str) -> Generator[list[str], None, None]:
    '''Load the HK spreadsheet with the given ID.'''
    fd, fn = tempfile.mkstemp(suffix='.csv')
    os.close(fd)
    try:
        gdown.download(id=spreadsheet_id, output=fn, format='csv')
        with open(fn, 'r') as io:
            reader = csv.reader(io)
            for row in reader: yield row
    finally:
        os.unlink(fn)


def _tag_from_string(tag_string: str) -> tuple[Tag, str]:
    '''Parse the tag string like "(0010,0020)" into a pydicom Tag; strip out HK's "clever" annotations.

    Returns the pydicom Tag and the cleaned tag string.
    
    >>> _tag_from_string('(0010,0020)')
    Tag(0x0010, 0x0020), '(0010,0020)'
    >>> _tag_from_string('(0010,0020) [private]')
    Tag(0x0010, 0x0020), '(0010,0020)'

    This may raise a ValueError if the tag string is invalid.
    '''
    tag_string = tag_string.replace('[private]', '').strip().strip('()')
    group, element = tag_string.split(',')
    return Tag(int(group, 16), int(element, 16)), tag_string


def _load_prescribed_tags(spreadsheet_id: str) -> dict:
    '''Load the prescribed tags from the Google Sheet with the given ID.'''
    tags, line_num = set(), 0
    for row in _load_hk_spreadsheet(spreadsheet_id):
        line_num += 1
        # Skip header row and annotation rows (i.e., rows with no tag) and HK's spreadsheet-wide grouping rows
        if row[0] == 'Set' or row[2] == '' or row[2] == '??': continue
        # Parse the tag string like "(0010,0020)" into a pydicom Tag; strip out HK's "clever" annotations
        try:
            dicom_tag, _ = _tag_from_string(row[2])
            tags.add(dicom_tag)
        except ValueError as e:
            _logger.error('❌ Ignoring invalid tag "%s" in row %d', row[2], line_num)
    return tags


def _top_level_data_elements(ds: pydicom.Dataset) -> Generator[pydicom.DataElement, None, None]:
    '''Yields only the top-level data elements in the given DICOM dataset wihtout recursing into subsequences.'''
    try:
        for elem in ds: yield elem
    except BytesLengthException:
        # There are bad tags in the good (and bad) DICOM files too, so skip 'em
        pass


def _record_value(tag: Tag, file_path: str, value: object, values: dict):
    '''Record the value in the values dictionary.'''

    # Transform it into something readable by HK
    if isinstance(value, bytes):
        value = '«binary data»'
    elif isinstance(value, Sequence):
        value = '«Sequence data»'
    else:  # It's a MultiValue, UID, PersonName, DSfloat, IS, list, float, int … or a str
        value = str(value)

    # Figure out which "site" it belongs to
    matches = _site_re.search(file_path)
    site = matches.group(1) if matches else '«unknown site»'

    # Record it
    sites = values.get(tag, dict())

    # HK thinks using the most prevalent value is "misleading" (🤔) so just gather sets of values
    # so we can pick one at random—no I don't get it either.
    # counter = sites.get(site, collections.Counter())
    # counter[value] += 1
    seen_values = sites.get(site, set())
    seen_values.add(value)
    sites[site] = seen_values
    values[tag] = sites


def _scan_file(file_path: str) -> tuple:
    '''Scan the DICOM header of the file at `file_path` and return results.'''
    # Remove excessive logging - only log errors
    try:
        ds = pydicom.dcmread(file_path, stop_before_pixels=True)
    except InvalidDicomError as e:
        # There are many non-DICOM files so just ignore these; no need to even log them
        return set(), {}

    tags, values = set(), {}
    
    for elem in _top_level_data_elements(ds):
        # Create a new object with updated state
        new_tag = ScannedTag(elem.tag)
        new_tag.in_file = True
        tags.add(elem.tag)
        _record_value(elem.tag, file_path, elem.value, values)
    
    return tags, values


def _find_all_files(folder: str):
    '''Find all the DICOM files in `folder`.'''
    for root, _, files in os.walk(folder):
        for fn in files:
            yield os.path.join(root, fn)


def _serial_scan(folder: str) -> tuple:
    '''Scan the DICOM headers of the files in `folder` serially.'''
    all_tags, all_values = set(), {}
    
    for file_path in _find_all_files(folder):
        tags, values = _scan_file(file_path)
        all_tags.update(tags)
        # Merge values dictionaries
        for tag, sites_dict in values.items():
            if tag not in all_values:
                all_values[tag] = {}
            for site, values in sites_dict.items():
                if site not in all_values[tag]:
                    all_values[tag][site] = set()
                all_values[tag][site].update(values)
    
    return all_tags, all_values


def _report_tag_appearance(tags: dict, values: dict):
    '''Report DICOM tag appearance in the files and spreadsheet to a CSV file.'''

    header = ['Tag', 'Name', 'In File', 'In Spreadsheet']

    with open('tag-appearance-report.csv', 'w') as io:
        writer = csv.writer(io)
        writer.writerow(header)
        for tag in sorted(tags.values(), key=lambda t: t.tag):
            row = [
                tag.tag, datadict.keyword_for_tag(tag.tag),
                '✓' if tag.in_file else '', '✓' if tag.in_spreadsheet else ''
            ]
            writer.writerow(row)


def _generate_hk_spreadsheet(tags: dict, values: dict, spreadsheet_id: str):
    '''Generate the HK spreadsheet with the given ID.'''

    sites = sorted(list(set([key for d in values.values() for key in d.keys()])))
    header = False
    with open('hk-spreadsheet-with-randomly-chosen-values.csv', 'w') as io:
        writer = csv.writer(io)
        for row in _load_hk_spreadsheet(spreadsheet_id):
            if not header and row[0] == 'Set':
                header = True
                header_row = row + [f'Randomly chosen value from DCM files in "{site}"' for site in sites]
                writer.writerow(header_row)
            elif row[2] == '':
                # This is one of HK's "clever" annotation rows, so just output it verbatim but add blanks for the
                # randonly chosen value columns for each site
                row_values = row + [''] * len(sites)
                writer.writerow(row_values)
            else:
                # This is a row with a tag, so we need to find the corresponding tag in the tags dictionary
                row_values = row
                try:
                    tag_obj, cleaned_tag_string = _tag_from_string(row[2])
                    possible_values_per_site = values.get(tag_obj, {})
                    for site in sites:
                        if site not in possible_values_per_site:
                            row_values += [f'🫙 No values found for tag "{cleaned_tag_string}" in DICOM file for site "{site}"']
                        else:
                            # Note that this could pick an empty string, but that's what HK wants I guess 🤪
                            row_values += [random.choice(list(possible_values_per_site[site]))]
                except ValueError as e:
                    row_values += [f'❌ Tag "{row[2]}" is an invalid DICOM tag ID'] * len(sites)
                writer.writerow(row_values)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('--version', action='version', version=VERSION)
    add_logging_argparse_options(parser)
    parser.add_argument(
        '--spreadsheet', default=_default_spreadsheet_id,
        help='ID of the Google Sheet to use; default "%(default)s"'
    )
    parser.add_argument('folders', nargs='+', help='Folders containing DICOM files to scan')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')

    # Map from a pydicom.tag.Tag to a ScannedTag object; and set of prescribed tags from the Google sheet
    prescribed_tags = _load_prescribed_tags(args.spreadsheet)

    _logger.info('🏁 Scanning %d folders: %s', len(args.folders), ', '.join(args.folders))
    time0 = time.monotonic()
    
    # Just use serial processing - it's faster and more reliable for I/O-bound tasks
    _logger.info('🔧 Using serial processing (multiprocessing disabled due to Cursor stupidity issues)')
    
    # Process all folders and combine results
    all_tags_set = set()
    all_values = {}
    
    for folder in args.folders:
        _logger.info('📁 Processing folder: %s', folder)
        tags_set, values = _serial_scan(folder)
        all_tags_set.update(tags_set)
        
        # Merge values dictionaries
        for tag, sites_dict in values.items():
            if tag not in all_values:
                all_values[tag] = {}
            for site, possible_values in sites_dict.items():
                if site not in all_values[tag]:
                    all_values[tag][site] = set()
                all_values[tag][site].update(possible_values)

    # Convert tags_set to tags dict with ScannedTag objects
    tags = {}
    for tag in all_tags_set:
        scanned_tag = ScannedTag(tag)
        scanned_tag.in_file = True
        tags[tag] = scanned_tag

    # Finally, mark all the tags in the spreadsheet and add any missing found in the spreadsheet but not in files
    for tag in prescribed_tags:
        existing_tag = tags.get(tag)
        if existing_tag is None:
            # Create new tag with in_spreadsheet=True
            new_tag = ScannedTag(tag)
            new_tag.in_spreadsheet = True
            tags[tag] = new_tag
        else:
            # Update existing tag
            existing_tag.in_spreadsheet = True

    _logger.info('🏁 Scanning took %s', humanize.naturaldelta(time.monotonic() - time0))
    _generate_hk_spreadsheet(tags, all_values, args.spreadsheet)
    _report_tag_appearance(tags, all_values)

    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: field usage report.

Tells what fields are in use by collections, datasets, and files in
Solr and marks those that appear in collection and dataset `.cfg` files.
'''

from .argparse import add_standard_argparse_options
import argparse, logging, requests, csv, os, glob, configparser

_logger = logging.getLogger(__name__)


def _find_specified_keys(metadata_dir: str, level: str) -> set[str]:
    if level == 'files':
        _logger.info('🙅 The files level does not have any specified keys')
        return set()
    elif level == 'collections':
        pattern = f'{metadata_dir}/*/*.cfg'
    elif level == 'datasets':
        pattern = f'{metadata_dir}/*/*/*.cfg'

    keys = set()
    for cfg_file in glob.glob(pattern):
        try:
            cfg = configparser.ConfigParser()
            cfg.optionxform = str
            _logger.debug('📖 Reading %s at %s level', cfg_file, level)
            cfg.read(cfg_file)
            for section in cfg.sections():
                keys.update(cfg[section].keys())
        except configparser.Error as ex:
            _logger.error(f'🚨 Error reading {cfg_file}; skipping it ({ex})')
    _logger.info('🔑 Found %d specified keys in %s', len(keys), level)
    return keys


def _report_fields_for_core(solr_url: str, core: str, metadata_dir: str):
    specified_keys = _find_specified_keys(metadata_dir, core)
    luke_url = solr_url if solr_url.endswith('/') else solr_url + '/'
    luke_url += f'{core}/admin/luke'
    params = {'numTerms': 0, 'wt': 'json'}
    response = requests.get(luke_url, params=params, verify=False)
    fields = response.json().get('fields', {})
    _logger.info('🌞 Found %d fields in Solr in %s', len(fields), core)
    with open(f'{core}.csv', 'w', newline='') as io:
        writer = csv.writer(io)
        writer.writerow(['Field', 'Count', 'Specified'])
        for field_name, info in sorted(fields.items()):
            doc_count = info.get('docs', 0)
            specified = '✓' if field_name in specified_keys else ''
            writer.writerow([field_name, doc_count, specified])


def _report_fields(solr_url: str, metadata_dir: str):
    for core in ('collections', 'datasets', 'files'):
        _report_fields_for_core(solr_url, core, metadata_dir)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument('metadata', help='Location of the metadata directory')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr_url = args.solr if args.solr.endswith('/') else args.solr + '/'
    _report_fields(solr_url, args.metadata)


if __name__ == '__main__':
    main()


# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: mangle DICOM headers and do other things.

To run: .venv/bin/mangle-headers BLINDED_PairID.to.Event.Identifier.with.BlindedID.csv data

See EDRN/EDRN-metadata#151 for the details.
'''

from . import VERSION
from .argparse import add_logging_argparse_options
from pathlib import Path
from .dcm_splitter import extract_bval
from pydicom.tag import Tag
from multiprocessing import Process, Queue, JoinableQueue
import argparse, sys, logging, pydicom, csv, re, time, humanize

_event_id_re = re.compile(r'/([0-9]{7})/')
_tag_to_delete = Tag((0x0400, 0x0561))  # "OriginalAttributesSequence", Adrian doesn't like it


_logger = logging.getLogger(__name__)
__doc__ = 'Mangle DICOM headers amongst other things'


def find_all_files(directory: str):
    '''Yield the full path to each file in `directory` and its subdirectories.'''
    root_path = Path(directory)
    for file_path in root_path.rglob("*"):
        if file_path.is_file():
            yield str(file_path)


def _load_event_ids(spreadsheet: str) -> dict:
    '''Load the event IDs to pair IDs plus visit code from the given spreadsheet.'''
    event_ids = {}
    with open(spreadsheet, 'r') as f:
        reader = csv.reader(f)
        for row in reader:
            if 'Image Event Identifier' in row[0]: continue
            event_ids[row[0]] = (row[1], row[2])
    return event_ids


def _mangle(event_ids: dict, file_path: str):
    '''Mangle the DICOM headers of the file file_path using information in event_ids.'''
    try:
        _logger.debug('🔬 Reading DICOM file %s', file_path)
        dcm = pydicom.dcmread(file_path)
        possible_event_id = _event_id_re.search(file_path)
        if possible_event_id is not None:
            event_id = possible_event_id.group(1)
        else:
            _logger.warning('🏟️ No event ID matched in file path %s; skipping', file_path)
            return
        blinded_shuffled_pair_id, visit_code = event_ids.get(event_id, (None, None))
        if blinded_shuffled_pair_id is None:
            _logger.warning('🦯 HEATHER: No event ID %s found in CSV for file %s; skipping it', event_id, file_path)
            return
        _logger.info(
            '🦧 Mangling DCM file %s with pair ID %s, event ID %s, and visit code %s',
            file_path, blinded_shuffled_pair_id, event_id, visit_code
        )

        # Mangle these headers
        dcm.PatientID = blinded_shuffled_pair_id
        dcm.StudyID = event_id
        dcm.ClinicalTrialTimePointID = visit_code
        dcm.InstitutionName = 'Anonymous'
        dcm.PatientName = blinded_shuffled_pair_id

        # Extract the b-value and save it
        bval, _ = extract_bval(dcm)
        _logger.debug('Got b-value %f', bval)
        if bval is not None:
            dcm.DiffusionBValue = bval

        # Delete the tag Adrian doesn't like
        if _tag_to_delete in dcm:
            del dcm[_tag_to_delete]

        # Save the mangled DICOM file
        _logger.debug('💾 Saving mangled DICOM file %s', file_path)
        dcm.save_as(file_path)
    except Exception as ex:
        _logger.error('🤷 Error mangling %s: %r; maybe not a DICOM file?', file_path, ex)


def _file_producer(queue: Queue, folder: str, concurrency: int):
    '''Produce the files to mangle.'''
    for file_path in find_all_files(folder):
        _logger.debug('🏭 Producer putting file %s', file_path)
        queue.put(file_path)
    _logger.debug('🔚 Producer putting Nones to end the consumers')
    for _ in range(concurrency): queue.put(None)


def _file_consumer(queue: Queue, event_ids: dict):
    '''Consume the files to mangle.'''
    while True:
        file_path = queue.get()
        if file_path is None:
            _logger.debug('🔚 Consumer got None; signaling task done')
            queue.task_done()
            break
        _logger.debug('🍽️ Consumer got file %s; mangling', file_path)
        _mangle(event_ids, file_path)
        queue.task_done()


def _mangle_parallel(event_ids: dict, folder: str, concurrency: int):
    '''Mangle the DICOM headers of the files in `folder` in parallel.'''
    queue = JoinableQueue()
    _logger.debug('🏁 Starting producer')
    producer = Process(target=_file_producer, args=(queue, folder, concurrency))
    producer.start()
    _logger.debug('🏁 Starting %d consumers', concurrency)
    consumers = [Process(target=_file_consumer, args=(queue, event_ids)) for _ in range(concurrency)]
    for c in consumers: c.start()
    _logger.debug('🔚 Joining producer')
    producer.join()
    _logger.debug('🔚 Joining consumers')
    queue.join()
    for c in consumers: c.join()


def _mangle_serial(event_ids: dict, folder: str):
    '''Mangle the DICOM headers of the files in `folder` in serial.'''
    for file_path in find_all_files(folder):
        _mangle(event_ids, file_path)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('--version', action='version', version=VERSION)
    parser.add_argument(
        '--concurrency', type=int, default=10,
        help='Number of concurrent processes to use; default is %(default)d'
    )
    add_logging_argparse_options(parser)
    parser.add_argument('spreadsheet', help='CSV file')
    parser.add_argument('folder', help='Folder containing DICOM files to mangle')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    event_ids = _load_event_ids(args.spreadsheet)

    time0 = time.monotonic()
    if args.concurrency > 1:
        _mangle_parallel(event_ids, args.folder, args.concurrency)
    elif args.concurrency == 1:
        _mangle_serial(event_ids, args.folder)
    else:
        raise ValueError(f'❌ Invalid concurrency: {args.concurrency}; must be 1 or greater')
    _logger.info('🏁 Mangling took %s', humanize.naturaldelta(time.monotonic() - time0))
    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: RDF handling.'''


import rdflib, logging

_logger = logging.getLogger(__name__)


def read_rdf(source: str) -> dict:
    '''Read the RDF at the given URL `source` and return a dictionary.

    The dictionary keys are RDF subject URIs. The values are dictionaries. Those dictionary's keys
    are predicate URIs, and the values are lists of objects.
    '''
    _logger.debug('Parsing RDF from %s', source)
    graph, statements = rdflib.Graph(), {}
    graph.parse(source)
    for statement, predicate, obj in graph:
        predicates = statements.get(statement, {})
        objs = predicates.get(predicate, [])
        objs.append(obj)
        predicates[predicate] = objs
        statements[statement] = predicates
    _logger.debug('Parsed %d statements', len(statements))
    return statements



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: delete dataset.

This script then assumes Solr is at https://localhost:8984/ (with a self-
signed certificate). You can override this, of course.

This will go through documents and delete datasets by given IDs.
While doing so it outputs a CSV file of all files that will need to
be removed from disk. The columns of this file are dataset ID,
file ID, and file path. The CSV goes to stdout.

It'll also delete all files under that dataset.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging, csv

_logger = logging.getLogger(__name__)

_disk, _s3 = '/labcas-data', 's3://edrn-labcas/archive'


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make any changes but show them instead"
    )
    parser.add_argument('datasetid', nargs='+', help='IDs of the datasets to delete (the id field)')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr = args.solr if args.solr.endswith('/') else args.solr + '/'
    datasets_solr = pysolr.Solr(solr + 'datasets', always_commit=True, verify=False)
    files_solr = pysolr.Solr(solr + 'files', always_commit=True, verify=False)

    _logger.debug('Opening CSV writer')
    writer = csv.writer(sys.stdout)
    writer.writerow([
        'Dataset ID', 'File ID', 'LabCAS Prepublished File Location', 'LabCAS Published File Location', 'S3 Location'
    ])
    for given_dataset_id in args.datasetid:
        _logger.info('Processing given dataset ID %s', given_dataset_id)

        for dataset in find_documents(datasets_solr, f'id:{given_dataset_id}*', ['id']):
            dataset_id = dataset['id']

            _logger.info('Finding files for dataset "%s"', dataset_id)
            file_ids = []
            for file in find_documents(files_solr, f'DatasetId:"{dataset_id}"', ['id', 'FileLocation', 'FileName']):
                pre, post = f'{_disk}/{file["id"]}', f'{file["FileLocation"]}/{file["FileName"]}'
                s3 = f'{_s3}/{file["id"]}'
                writer.writerow([dataset_id, file['id'], pre, post, s3])
                file_ids.append(file['id'])

            _logger.info('Done finding files for %s, found %d; dry run=%r', dataset_id, len(file_ids), args.dryrun)
            if file_ids:
                _logger.info("Deleting (%r) from Solr: %r", not args.dryrun, ','.join(file_ids))
                if not args.dryrun:
                    files_solr.delete(id=file_ids)
                _logger.info("Deleting (%r) from Solr: %s", not args.dryrun, dataset_id)

            _logger.info('Now to delete (%r) the dataset %s', not args.dryrun, dataset_id)
            if not args.dryrun:
                datasets_solr.delete(id=dataset_id)

    sys.exit(0)


if __name__ == '__main__':
    main()



'''🛠️ JPL LabCAS Utilities: fix OwnerPrincipal in a collection.

This will take an existing document and replace a list field with a new
items as given on the command line.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging, os.path, configparser


_logger = logging.getLogger(__name__)
__doc__ = 'Fix the OwnerPrincipal field in a LabCAS collection and its datasets and files'
_max_updates = 100


def _read_owner_principals(cfgfile: str, section: str) -> list[str]:
    '''Read the `cfgfile` and look in the `section` for `OwnerPrincipal` and return
    a list of each one found.
    '''
    if not os.path.isfile(cfgfile):
        raise ValueError(f"Expected {cfgfile} to be a file that exists but it doesn't")
    cp = configparser.ConfigParser()
    cp.read(cfgfile)
    if not cp.has_section(section):
        raise ValueError(f'Expected {cfgfile} to have {section} but it only has {cp.sections()}')
    try:
        principals = cp.get(section, 'OwnerPrincipal')
        return principals.split('|')
    except configparser.NoOptionError:
        _logger.warning('⚠️ %s config file %s has no OwnerPrincipal, assuming none', section, cfgfile)
        return []


def _apply_updates(solr: pysolr.Solr, updates: list, dryrun: bool = True):
    if dryrun:
        _logger.info('🌵 DRY RUN: not calling update on %d updates', len(updates))
    else:
        _logger.debug('😎 HOT RUN: applying updates %r', updates)
        solr.add(updates)


def _set_owner_principals(solr_url: str, core: str, query: str, principals: list[str], dryrun: bool):
    solr_url = f'{solr_url}{core}'
    solr = pysolr.Solr(solr_url, always_commit=True, verify=False)
    updates = []
    for doc in find_documents(solr, query, ['id']):
        updates.append({'id': doc['id'], 'OwnerPrincipal': {'set': principals}})
        if len(updates) >= _max_updates:
            _apply_updates(solr, updates, dryrun)
            updates = []
    if len(updates) > 0:
        _apply_updates(solr, updates, dryrun)


def _fix_owner_principals_for_collection_dir(solr_url: str, collection_dir: str, dryrun: bool = True):
    collection_name = os.path.basename(collection_dir)
    _logger.info(
        '🧑‍🔧 Fixing OwnerPrincipal for collection %s in Solr at %s; dryrun=%d',
        collection_name, solr_url, dryrun
    )

    # First update the collection
    collection_cfg = os.path.abspath(os.path.join(collection_dir, f'{collection_name}.cfg'))
    collection_principals = _read_owner_principals(collection_cfg, 'Collection')
    if len(collection_principals) == 0:
        _logger.info('No OwnerPrincipals at Collection level, so not making any changes to collection')
    else:
        _logger.info(
            'At the Collection %s level, setting owners to %r', collection_name, collection_principals 
        )
        _set_owner_principals(
            solr_url, 'collections', f'id:{collection_name}', collection_principals, dryrun
        )

    # Now find each subdirectory and update the datasets and files. Note this assumes
    # there are no nested dataset directories, which is the case as of 2025-02-07 in
    # EDRN-metadata.
    for dirpath, dirnames, filenames in os.walk(collection_dir):
        for fn in filenames:
            if fn.endswith('.cfg'):
                cfg = os.path.abspath(os.path.join(dirpath, fn))
                if cfg == collection_cfg: continue
                dataset_name = os.path.basename(dirpath)
                dataset_principals = _read_owner_principals(cfg, 'Dataset')
                query = f'id:{collection_name}/{dataset_name}*'
                _logger.info(
                    'At the Dataset %s level, setting owners to %r', dataset_name, dataset_principals
                )
                _set_owner_principals(solr_url, 'datasets', query, dataset_principals, dryrun)
                _logger.info(
                    'For Files under %s, setting owners to %r', dataset_name, dataset_principals
                )
                _set_owner_principals(solr_url, 'files', query, dataset_principals, dryrun)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make changes, show them instead"
    )
    parser.add_argument('dir', help='Path to directory with collection `.cfg` file and dataset subidrs')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    collection_dir = args.dir if not args.dir.endswith('/') else args.dir[:-1]
    solr_url = args.solr if args.solr.endswith('/') else args.solr + '/'

    if not os.path.isdir(collection_dir):
        raise ValueError(f'Expecting {collection_dir} to be a directory')
    _fix_owner_principals_for_collection_dir(solr_url, collection_dir, args.dryrun)

    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: S3 report.

This produces a CSV as outlined in `jpl-labcas/jpl.labcas.utils#1`.
'''

from . import VERSION
from .argparse import add_logging_argparse_options
from enum import Enum
from urllib.parse import urlparse
import argparse, logging, boto3, os, os.path, csv, sys

_logger = logging.getLogger(__name__)


class _ObjectType(Enum):
    FILE = 0
    FOLDER = 1


class _Folder:
    '''A "folder" in S3, tracking the number of files it contains and its child folders.'''

    def __init__(self, path):
        '''Initialize a "folder" giving its `path`.'''
        self.path = path
        self.files = 0
        self.children = []

    def average_num_child_files(self):
        '''Tell the average number of files in immediate children.

        This can return a number or None if the folder has no children folders.
        '''
        num_children = len(self.children)
        return None if num_children == 0 else sum([i.files for i in self.children]) / num_children

    def __repr__(self):
        return f'_Folder(path={self.path},files={self.files},children={self.children})'


def _extract_bucket_path(s3_url: str) -> tuple[str, str]:
    '''Given an S3 URL, return the name of the bucket and the path of the folder within
    the bucket.

    For example, `s3://blah/foo/bar` would give `('blah', 'foo/bar')`.
    '''
    parsed = urlparse(s3_url)
    return parsed.netloc, parsed.path.lstrip('/')


def _generate_s3_contents(s3, bucket, base_path):
    '''Generate the contents of everthing in `base_path` in the S3 `bucket`.

    This handles pagination.
    '''
    continuation_token = None
    while True:
        request_params = {'Bucket': bucket, 'Prefix': base_path}
        if continuation_token:
            request_params['ContinuationToken'] = continuation_token
        response = s3.list_objects_v2(**request_params)
        if 'Contents' in response:
            for obj in response['Contents']:
                path = obj['Key']
                if obj['Size'] == 0 and path.endswith('/'):
                    yield _ObjectType.FOLDER, path
                else:
                    yield _ObjectType.FILE, path
        if response.get('IsTruncated'):
            continuation_token = response['NextContinuationToken']
        else:
            break


def _parent(kind, path):
    '''For the `kind` of object at `path` give the path to its parent.

    The parent is always a folder so will always end with a `/`.
    '''
    if kind == _ObjectType.FOLDER:
        return os.path.dirname(path[0:-1]) + '/'
    else:
        return os.path.dirname(path) + '/'


def _generate_report(s3, s3_url):
    '''Using the given `s3` client, generate a report for the contents of `s3_url`.

    The report is written to the standard output as a CSV file with 4 columns
    as specified in EDRN/jpl.labcas.utils#1.
    '''
    bucket, root_path = _extract_bucket_path(s3_url)
    root, folders, count = None, {}, 0
    for kind, path in _generate_s3_contents(s3, bucket, root_path):
        _logger.debug('Kind = %s, path = %s', kind, path)
        parent_path = _parent(kind, path)
        if kind == _ObjectType.FOLDER:
            folder = _Folder(path)
            folders[path] = folder
            if path == root_path:
                assert root is None
                root = folder
            else:
                folders[parent_path].children.append(folder)
        else:  # must be FILE
            folders[parent_path].files += 1
        count += 1
        if count % 1000 == 0: _logger.info('Processed %d entries so far', count)

    writer = csv.writer(sys.stdout)
    writer.writerow(('folder path', '# files', '# sub-folders', 'avg # of files in sub-folders'))
    for folder in folders.values():
        avg = folder.average_num_child_files()
        avg = f'{avg:.2f}' if avg else ''
        writer.writerow((folder.path, folder.files, len(folder.children), avg))
    sys.stdout.flush()


def main():
    '''Entrypoint.'''
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('--version', action='version', version=VERSION)
    parser.add_argument('--profile', help='AWS profile to use, optional; set AWS_PROFILE environment variable if needed')
    parser.add_argument('s3url', help='URL to report on in the form s3://BUCKET/FOLDER/FOLDER/…')
    add_logging_argparse_options(parser)
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    profile = args.profile if args.profile else os.getenv('AWS_PROFILE')
    session = boto3.Session(profile_name=profile)  # Assume AWS_PROFILE is set
    s3_client = session.client('s3')
    _generate_report(s3_client, args.s3url)


if __name__ == '__main__':
    main()



'''🛠️ JPL LabCAS Utilities: report missing event IDs.

Given event IDs, report which are missing in LabCAS Solr.

Typical usage: .venv/bin/missing-event-ids 1635113 9353978 xxxyyzz
Or: awk -F, '{print $1}' BLINDED_PairID.to.Event.Identifier.with.BlindedID.csv | xargs .venv/bin/missing-event-ids > missing-event-ids.txt

This writes to stdout the event IDs that are not found in the files core in Solr.

Supports https://github.com/EDRN/EDRN-metadata/issues/151#issuecomment-3215362878 (this comment specifically under "Add new verification).
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging, os.path, json, re


_logger = logging.getLogger(__name__)
__doc__ = 'Report missing event IDs'


def _report_missing_event_ids(solr: pysolr.Solr, event_ids: list[str]):
    for event_id in event_ids:
        _logger.info('🔍 Checking for event ID %s', event_id)
        query = f'eventID:{event_id}'
        results = solr.search(query, rows=0)
        if results.hits == 0:
            print(event_id)
        else:
            _logger.info('🔍 %d results found for event ID %s', results.hits, event_id)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument('event_ids', nargs='+', help='The event IDs to check')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr_url = args.solr if args.solr.endswith('/') else args.solr + '/'
    solr = pysolr.Solr(solr_url + 'files', always_commit=True, verify=False)
    event_ids = sorted(list(set(args.event_ids)))
    _report_missing_event_ids(solr, event_ids)
    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: assign UUIDs.

This script then assumes Solr is at https://localhost:8984/ (with a self-
signed certificate). You can override this, of course.

This will go through documents and, if there isn't a UUID element, will
assign one.
'''

from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging, uuid

_logger = logging.getLogger(__name__)

_disk, _s3 = '/labcas-data', 's3://edrn-labcas/archive'


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument(
        '-r', '--dryrun', action='store_true', help="Dry run; don't make any changes but show them instead"
    )
    parser.add_argument('core', help='What Solr core to use (collections, datasets, files, etc.)')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    solr_url = f'{args.solr}{args.core}' if args.solr.endswith('/') else f'{args.solr}/{args.core}'
    _logger.debug('Solr URL is %s', solr_url)
    solr = pysolr.Solr(solr_url, always_commit=True, verify=False)

    count = 0
    for doc in find_documents(solr, '*:*', None):
        existing_id = doc.get('uuid', None)
        if not existing_id:
            new_id = uuid.uuid4().urn  # Fully random UUID
            _logger.debug('Doc %s lacks UUID, assigning %s', doc['id'], new_id)
            doc['uuid'] = new_id
            del doc['_version_']
            count += 1
            if args.dryrun:
                _logger.info('Dry run: would be updating doc %s', doc['id'])
            else:
                solr.add(doc)
            if count % 1000 == 0:
                _logger.info(f'Processed {"(not really, dry run)" if args.dryrun else ""} %d so far', count)
        else:
            _logger.debug('Doc %s already has UUID %s', doc['id'], existing_id)

    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: common prefixes.

Find common prefixes in FileLocation fields in Solr.
'''

from .argparse import add_standard_argparse_options
import requests, logging, argparse, os.path


_logger = logging.getLogger(__name__)

# Possible query?
# https://localhost:8984/solr/files/select?q=*:*&rows=0&facet=true&facet.field=FileLocation&facet.limit=-1&facet.sort=count

def _print_common_prefixes(url: str):
    url = url + 'files/select'
    _logger.info('Finding common prefixes in %s', url)
    params = {
        'q': '*:*', 'rows': 0, 'wt': 'json',
        'facet': 'true', 'facet.field': 'FileLocation', 'facet.limit': '-1', 'facet.sort': 'count'
    }
    response = requests.get(url, params=params, headers={'Accept': 'application/json'}, verify=False)
    response.raise_for_status()
    result = response.json()
    file_locations = result['facet_counts']['facet_fields']['FileLocation'][::2]
    for f in file_locations:
        print(f)


def main():
    parser = argparse.ArgumentParser(__doc__)
    add_standard_argparse_options(parser)
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    url = args.solr if args.solr.endswith('/') else args.solr + '/'
    _print_common_prefixes(url)


if __name__ == '__main__':
    main()


# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: generate various reports.'''


from .argparse import add_standard_argparse_options
from .solr import find_documents
import argparse, sys, pysolr, logging, csv, requests, os.path, re, collections, pydicom

_logger = logging.getLogger(__name__)
_event_id_re = re.compile(r'/([0-9]{7})/')


class Report:
    def execute(self, args: argparse.Namespace):
        raise NotImplementedError('🤪 Subclasses must implement this method')


class EventCorrelationReport(Report):
    def execute(self, args: argparse.Namespace):
        '''Correlate event IDs in Solr with those in the given CSV file.'''

        # Gather all the event IDs in the CSV file
        suzanna_event_ids = dict()
        with open(args.csv, 'r', newline='') as io:
            reader = csv.reader(io)
            for row in reader:
                if row[0] == 'STUDY_PARTICIPANT_ID': continue  # skip header
                # For Prostate_MRI `pmri.csv`:
                # ppt_id, event_id, site_id = row
                # For Lung_Team_Project_2 `ltp2.csv`:
                ppt_id, event_id = row
                site_id = ppt_id[0:3]
                suzanna_event_ids[event_id] = (site_id, ppt_id)

        # Gather all the event IDs for Prostate_MRI in Solr
        blindings, solr, files = dict(), pysolr.Solr(args.solr + 'files', verify=False), 0
        # for doc in find_documents(solr, 'CollectionId:Prostate_MRI', ['eventID', 'BlindedSiteID']):
        for doc in find_documents(solr, 'CollectionId:Lung_Team_Project_2', ['eventID', 'BlindedSiteID']):
            files += 1
            if files % 1000 == 0: _logger.info('Read through %d files', files)
            event_id = doc.get('eventID', [None])[0]
            if not event_id: continue  # skipping those without event IDs
            site_id = doc.get('BlindedSiteID', [None])[0]
            if not site_id: continue  # skip those without blinded site IDs
            blindings[event_id] = site_id

        only_in_suzanna = set(suzanna_event_ids.keys()) - set(blindings.keys())
        only_in_solr = set(blindings.keys()) - set(suzanna_event_ids.keys())

        writer = csv.writer(sys.stdout)
        writer.writerow(['Event ID', 'Only in Suzanna Site ID', 'Participant ID'])
        for e in only_in_suzanna:
            writer.writerow([e, suzanna_event_ids[e][0], suzanna_event_ids[e][1]])
        for e in only_in_solr:
            raise ValueError('should not happen')
            # writer.writerow([e, f'Only in Solr for site {blindings[e]}', '«unknown»'])


class PatientIDsReport(Report):
    def execute(self, args: argparse.Namespace):
        if args.folder is None:
            raise ValueError('🤷‍♂️ Folder is required for patientids report')
        event_ids = collections.defaultdict(set)
        for dirpath, _, filenames in os.walk(args.folder):
            for filename in filenames:
                file_path = os.path.join(dirpath, filename)
                possible_event_id = _event_id_re.search(file_path)
                if possible_event_id is not None:
                    event_id = possible_event_id.group(1)
                    try:
                        dcm = pydicom.dcmread(file_path)
                        event_ids[event_id].add(dcm.PatientID)
                        print(f'🔬 For event ID {event_id} found patient ID {dcm.PatientID} in {file_path}')
                    except Exception as ex:
                        _logger.warning('🤷‍♂️ Error reading as DICOM file %s: %r', file_path, ex)
                        continue
                else:
                    _logger.warning('🏟️ No event ID matched in file path %s; skipping', file_path)
                    continue

        writer = csv.writer(sys.stdout)
        writer.writerow(['Event ID', 'Patient IDs'])
        for event_id, patient_ids in event_ids.items():
            writer.writerow([event_id, '|'.join(patient_ids)])


class AvailabilityReport(Report):
    def _get_file_path(self, doc: dict) -> str:
        '''This reproduces the logic in `DownloadServiceImpl.java` in the LabCAS backend.'''

        # LabCAS Solr is so weird; `FileLocation` and `FileName` are strings, but
        # `name` is an array of strings.
        file_directory = doc.get('FileLocation', None)
        if file_directory is None: return None
        file_name, name = doc.get('FileName', None), doc.get('name', [None])[0]
        if name is not None and len(name) > 0:
            return os.path.join(file_directory, name)
        elif file_name is not None and len(file_name) > 0:
            return os.path.join(file_directory, file_name)
        else:
            return None

    def execute(self, args: argparse.Namespace):
        with open('missing.csv', 'w', newline='') as io:
            writer = csv.writer(io)
            writer.writerow(['LabCAS ID', 'File Path'])
            solr = pysolr.Solr(args.solr + 'files', verify=False)
            for doc in find_documents(solr, '*:*', ['FileLocation', 'FileName', 'name', 'id']):
                labcas_id, file_path = doc.get('id', None), self._get_file_path(doc)
                if labcas_id is None:
                    _logger.warning('No ID field for %r', doc)
                    continue
                if not file_path:
                    _logger.warning('No file path for %s', id)
                    continue
                if not os.path.isfile(file_path):
                    writer.writerow([labcas_id, file_path])


class EventIDReport(Report):
    def execute(self, args: argparse.Namespace):
        # Make a dict that maps from blinded site ID to a set of event IDs
        _collections_with_event_IDs = [
            'Lung_Team_Project_2',
            'Prostate_MRI'
        ]
        blindings, solr, files = dict(), pysolr.Solr(args.solr + 'files', verify=False), 0
        for collection in _collections_with_event_IDs:
            for doc in find_documents(solr, f'CollectionId:{collection}', ['eventID', 'BlindedSiteID']):
                files += 1
                if files % 1000 == 0: _logger.info('Processed %d files', files)
                event_id = doc.get('eventID', [None])[0]
                if not event_id: continue  # skipping those without event IDs
                blind = doc.get('BlindedSiteID', [None])[0]
                if not blind: continue  # same without blinding
                events_collection_pairs = blindings.get(blind, set())
                pair = (event_id, collection)
                events_collection_pairs.add(pair)
                blindings[blind] = events_collection_pairs

        writer = csv.writer(sys.stdout)
        writer.writerow(['Blinded Site ID', 'Collection ID', 'Event ID'])
        for blinded_site_id, events in blindings.items():
            for events_collection_pairs in events:
                writer.writerow([blinded_site_id, events_collection_pairs[1], events_collection_pairs[0]])

class PrivacyReport(Report):
    def execute(self, args: argparse.Namespace):
        _privacy_fields = [
            'dicom_EthnicGroup',
            'dicom_Occupation',
            'dicom_PatientAddress',
            'dicom_PatientBirthDate',
            'dicom_PatientBirthName',
            'dicom_PatientID',
            'dicom_PatientMotherBirthName',
            'dicom_PatientName',
            'dicom_PatientReligiousPreference',
            'dicom_PatientSex',
            'dicom_PatientTelephoneNumbers',
            'labcas.dicom:BirthDate',
            'labcas.dicom:ID',
            'labcas.dicom:Name',
            'labcas.dicom:PatientAddress',
            'labcas.dicom:Sex',
        ]
        _max_markdown = 15
        solr = pysolr.Solr(args.solr + 'files', verify=False)  # noqa
        with open('privacy.csv', 'w', newline='') as csvio, open('privacy.md', 'w') as mdio:
            writer = csv.writer(csvio)
            writer.writerow(('Field', 'Count', 'Value'))
            for field_name in _privacy_fields:
                # Trying the "Luke" handler for now; see
                # https://solr.apache.org/guide/8_7/luke-request-handler.html
                luke_url = f'{args.solr}files/admin/luke?numTerms=999999&wt=json&fl={field_name}'
                results = requests.get(luke_url, verify=False)
                try:
                    counts = results.json()['fields'][field_name]['topTerms']
                except KeyError:
                    continue

                # This gives different results:
                # results = solr.search(q='*:*', rows=0, facet='true', **{
                #     'facet.field': field_name,
                #     'facet.sort': 'count',
                #     'facet.limit': -1
                # })
                # counts = results.facets['facet_fields'][field_name]

                for v, c in zip(counts[0::2], counts[1::2]):
                    writer.writerow((field_name, c, v))
                counts = [f'`{v}`: {c}' for v, c in zip(counts[0::2], counts[1::2])]
                if len(counts) == 0:
                    print(f'- {field_name}: none', file=mdio)
                elif len(counts) > _max_markdown:
                    print(f'- {field_name}: {", ".join(counts[:_max_markdown])} …', file=mdio)
                else:
                    print(f'- {field_name}: {", ".join(counts)}', file=mdio)


# Register the reports by name
_reports = {
    'events': EventIDReport,
    'privacy': PrivacyReport,
    'eventcor': EventCorrelationReport,
    'availability': AvailabilityReport,
    'patientids': PatientIDsReport,
}

def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    parser.add_argument('--csv', help="Suzanna's CSV file of event IDs in column 2 for eventcor report")
    parser.add_argument('--core', help='What Solr core to use (collections, datasets, files, etc.) for the field report')
    parser.add_argument('report', choices=_reports.keys(), help='What kind of report to make')
    parser.add_argument('folder', nargs='?', help='Folder to process for patientids report')
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    report = _reports.get(args.report)
    if not report:
        raise ValueError(f'Unknown report: {args.report}')
    report().execute(args)
    sys.exit(0)


if __name__ == '__main__':
    main()



# encoding: utf-8

'''🛠️ JPL LabCAS Utilities: backup Solr metadata for LabCAS.

This just runs the backup endpoint on the three LabCAS cores, `collections`, `datasets`, and `files`.

By default it'll run on the default Solr (https://localhost:8984/solr/) but you can use `--solr` to
override that; for example, if you've tunneled `labcas-dev`'s Solr onto port 8987, you can specify
`--solr https://localhost:8987/solr/`.
'''

from .argparse import add_standard_argparse_options
import argparse, sys, logging, requests


_logger = logging.getLogger(__name__)


def _backup(url: str, core: str):
    _logger.info('Backing up core «%s» on %s', core, url)
    endpoint = f'{url}{core}/replication'
    _logger.debug('Replication endpoint is %s', endpoint)
    response = requests.get(
        endpoint, params={'command': 'backup', 'wt': 'json'}, headers={'Accept': 'application/json'}, verify=False
    )
    response.raise_for_status()
    result = response.json()
    _logger.info('Backup status: %s', result.get('status'))


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add_standard_argparse_options(parser)
    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel, format='%(levelname)s %(message)s')
    url = args.solr if args.solr.endswith('/') else args.solr + '/'
    for core in ('collections', 'datasets', 'files'):
        _backup(url, core)
    sys.exit(0)


if __name__ == '__main__':
    main()
